{-# OPTIONS_GHC -Wall #-}
module Debian.Install
       ( File
       , Flag(..)
       , withFlags
       , install
       ) where

import System.Directory (createDirectoryIfMissing)
import System.FilePath (takeDirectory)
import qualified System.Posix.Files as F
import System.Posix.Types (FileMode)
import System.Process (readProcess)

type File = (FilePath, [String])

data Flag
  = Mode FileMode
  | Strip
  | D

-- | Bracket some file manipulation function with options like those
-- provided by install(1).
withFlags :: (FilePath -> IO a)  -- ^ The action
          -> [Flag]
          -> FilePath         -- ^ The file that will be modified
          -> IO a
withFlags action flags path =
  mapM_ d flags >>
  action path >>= \ result ->
  mapM_ strip flags >>
  mapM_ mode flags >>
  return result
  where
    d D = (createDirectoryIfMissing True (takeDirectory path))
    d _ = return ()
    strip Strip = readProcess "/usr/bin/strip" [path] "" >> return ()
    strip _ = return ()
    mode (Mode m) = F.setFileMode path m
    mode _ = return ()

-- | Like install(1), create a file and set some of its attributes
-- like mode.
install :: [Flag] -> FilePath -> [String] -> IO ()
install flags path text = withFlags (`writeFile` (unlines text)) flags path

{-
mode755 :: FileMode
mode755 =
  foldr F.unionFileModes F.nullFileMode [F.ownerReadMode, F.ownerWriteMode, F.ownerExecuteMode,
                                         F.groupReadMode, F.groupExecuteMode,
                                         F.otherReadMode, F.otherExecuteMode]

-- | Create some directory in a package.  (Obsolete, use debianDirs)
mkdir :: String -> FilePath -> IO ()
mkdir package dir =
  createDirectoryIfMissing True ("debian" </> package </> dir)

-- | Copy the executable from the build directory of an eponymous
-- package to the executable install directory, set mode and strip.
copyExecutable :: [Flag] -> String -> FilePath -> IO ()
copyExecutable flags dist name =
  withFlags (copyFile src) ([D, Mode mode755, Strip] ++ flags) dst
  where
    src = dist </> "build" </> name </> name
    dst = "debian" </> name </> "usr/bin" </> name
-}